@@ -217,7 +217,9 @@ def find_group_cohorts(labels, chunks, merge=True, method="cohorts"):
         return chunks_cohorts.values()
 
 
-def rechunk_for_cohorts(array, axis, labels, force_new_chunk_at, chunksize=None):
+def rechunk_for_cohorts(
+    array, axis, labels, force_new_chunk_at, chunksize=None, ignore_old_chunks=False, debug=False
+):
     """
     Rechunks array so that each new chunk contains groups that always occur together.
 
@@ -257,6 +259,9 @@ def rechunk_for_cohorts(array, axis, labels, force_new_chunk_at, chunksize=None)
     force_new_chunk_at = _atleast_1d(force_new_chunk_at)
     oldchunks = array.chunks[axis]
     oldbreaks = np.insert(np.cumsum(oldchunks), 0, 0)
+    if debug:
+        labels_at_breaks = labels[oldbreaks[:-1]]
+        print(labels_at_breaks[:40])
 
     isbreak = np.isin(labels, force_new_chunk_at)
     if not np.any(isbreak):
@@ -276,13 +281,19 @@ def rechunk_for_cohorts(array, axis, labels, force_new_chunk_at, chunksize=None)
         else:
             next_break_is_close = False
 
-        if idx in oldbreaks or (counter >= chunksize and not next_break_is_close):
+        if (not ignore_old_chunks and idx in oldbreaks) or (
+            counter >= chunksize and not next_break_is_close
+        ):
             divisions.append(idx)
             counter = 1
             continue
         counter += 1
 
     divisions.append(len(labels))
+    if debug:
+        labels_at_breaks = labels[divisions[:-1]]
+        print(labels_at_breaks[:40])
+
     newchunks = tuple(np.diff(divisions))
     assert sum(newchunks) == len(labels)
 
@@ -429,7 +440,7 @@ def factorize_(
                 else:
                     sorter = None
                 idx = np.searchsorted(expect, groupvar.ravel(), sorter=sorter)
-                mask = isnull(groupvar.ravel())
+                mask = isnull(groupvar.ravel()) | (idx == len(expect))
                 # TODO: optimize?
                 idx[mask] = -1
                 if not sort:
@@ -498,6 +509,7 @@ def chunk_argreduce(
     dask.array.reductions.argtopk
     """
     array, idx = array_plus_idx
+    by = np.broadcast_to(by, array.shape)
 
     results = chunk_reduce(
         array,
@@ -511,10 +523,13 @@ def chunk_argreduce(
         sort=sort,
     )
     if not isnull(results["groups"]).all():
-        # will not work for empty groups...
-        # glorious
         idx = np.broadcast_to(idx, array.shape)
+
+        # array, by get flattened to 1D before passing to npg
+        # so the indexes need to be unraveled
         newidx = np.unravel_index(results["intermediates"][1], array.shape)
+
+        # Now index into the actual "global" indexes `idx`
         results["intermediates"][1] = idx[newidx]
 
     if reindex and expected_groups is not None:
@@ -522,6 +537,8 @@ def chunk_argreduce(
             results["intermediates"][1], results["groups"].squeeze(), expected_groups, fill_value=0
         )
 
+    assert results["intermediates"][0].shape == results["intermediates"][1].shape
+
     return results
 
 
@@ -696,6 +713,7 @@ def _finalize_results(
     axis: Sequence[int],
     expected_groups: pd.Index | None,
     fill_value: Any,
+    reindex: bool,
 ):
     """Finalize results by
     1. Squeezing out dummy dimensions
@@ -730,7 +748,7 @@ def _finalize_results(
             finalized[agg.name] = np.where(count_mask, fill_value, finalized[agg.name])
 
     # Final reindexing has to be here to be lazy
-    if expected_groups is not None:
+    if not reindex and expected_groups is not None:
         finalized[agg.name] = reindex_(
             finalized[agg.name], squeezed["groups"], expected_groups, fill_value=fill_value
         )
@@ -749,10 +767,11 @@ def _aggregate(
     axis: Sequence,
     keepdims,
     fill_value: Any,
+    reindex: bool,
 ) -> FinalResultsDict:
     """Final aggregation step of tree reduction"""
     results = combine(x_chunk, agg, axis, keepdims, is_aggregate=True)
-    return _finalize_results(results, agg, axis, expected_groups, fill_value)
+    return _finalize_results(results, agg, axis, expected_groups, fill_value, reindex)
 
 
 def _expand_dims(results: IntermediateDict) -> IntermediateDict:
@@ -866,34 +885,45 @@ def _grouped_combine(
         array_idx = tuple(
             _conc2(x_chunk, key1="intermediates", key2=idx, axis=axis) for idx in (0, 1)
         )
-        results = chunk_argreduce(
-            array_idx,
-            groups,
-            func=agg.combine[slicer],  # count gets treated specially next
-            axis=axis,
-            expected_groups=None,
-            fill_value=agg.fill_value["intermediate"][slicer],
-            dtype=agg.dtype["intermediate"][slicer],
-            engine=engine,
-            sort=sort,
-        )
+
+        # for a single element along axis, we don't want to run the argreduction twice
+        # This happens when we are reducing along an axis with a single chunk.
+        avoid_reduction = array_idx[0].shape[axis[0]] == 1
+        if avoid_reduction:
+            results = {"groups": groups, "intermediates": list(array_idx)}
+        else:
+            results = chunk_argreduce(
+                array_idx,
+                groups,
+                func=agg.combine[slicer],  # count gets treated specially next
+                axis=axis,
+                expected_groups=None,
+                fill_value=agg.fill_value["intermediate"][slicer],
+                dtype=agg.dtype["intermediate"][slicer],
+                engine=engine,
+                sort=sort,
+            )
 
         if agg.chunk[-1] == "nanlen":
             counts = _conc2(x_chunk, key1="intermediates", key2=2, axis=axis)
-            # sum the counts
-            results["intermediates"].append(
-                chunk_reduce(
-                    counts,
-                    groups,
-                    func="sum",
-                    axis=axis,
-                    expected_groups=None,
-                    fill_value=(0,),
-                    dtype=(np.intp,),
-                    engine=engine,
-                    sort=sort,
-                )["intermediates"][0]
-            )
+
+            if avoid_reduction:
+                results["intermediates"].append(counts)
+            else:
+                # sum the counts
+                results["intermediates"].append(
+                    chunk_reduce(
+                        counts,
+                        groups,
+                        func="sum",
+                        axis=axis,
+                        expected_groups=None,
+                        fill_value=(0,),
+                        dtype=(np.intp,),
+                        engine=engine,
+                        sort=sort,
+                    )["intermediates"][0]
+                )
 
     elif agg.reduction_type == "reduce":
         # Here we reduce the intermediates individually
@@ -959,7 +989,7 @@ def split_blocks(applied, split_out, expected_groups, split_name):
     return intermediate, group_chunks
 
 
-def _reduce_blockwise(array, by, agg, *, axis, expected_groups, fill_value, engine, sort):
+def _reduce_blockwise(array, by, agg, *, axis, expected_groups, fill_value, engine, sort, reindex):
     """
     Blockwise groupby reduction that produces the final result. This code path is
     also used for non-dask array aggregations.
@@ -989,29 +1019,15 @@ def _reduce_blockwise(array, by, agg, *, axis, expected_groups, fill_value, engi
         kwargs=finalize_kwargs,
         engine=engine,
         sort=sort,
+        reindex=reindex,
     )  # type: ignore
 
     if _is_arg_reduction(agg):
-        if array.ndim > 1:
-            # default fill_value is -1; we can't unravel that;
-            # so replace -1 with 0; unravel; then replace 0 with -1
-            # UGH!
-            idx = results["intermediates"][0]
-            mask = idx == agg.fill_value["numpy"][0]
-            idx[mask] = 0
-            # Fix npg bug where argmax with nD array, 1D group_idx, axis=-1
-            # will return wrong indices
-            idx = np.unravel_index(idx, array.shape)[-1]
-            idx[mask] = agg.fill_value["numpy"][0]
-            results["intermediates"][0] = idx
-    elif agg.name in ["nanvar", "nanstd"]:
-        # TODO: Fix npg bug where all-NaN rows are 0 instead of NaN
-        value, counts = results["intermediates"]
-        mask = counts <= 0
-        value[mask] = np.nan
-        results["intermediates"][0] = value
+        results["intermediates"][0] = np.unravel_index(results["intermediates"][0], array.shape)[-1]
 
-    result = _finalize_results(results, agg, axis, expected_groups, fill_value=fill_value)
+    result = _finalize_results(
+        results, agg, axis, expected_groups, fill_value=fill_value, reindex=reindex
+    )
     return result
 
 
@@ -1085,7 +1101,9 @@ def dask_groupby_agg(
     )
     if method == "blockwise":
         #  use the "non dask" code path, but applied blockwise
-        blockwise_method = partial(_reduce_blockwise, agg=agg, fill_value=fill_value)
+        blockwise_method = partial(
+            _reduce_blockwise, agg=agg, fill_value=fill_value, reindex=reindex
+        )
     else:
         # choose `chunk_reduce` or `chunk_argreduce`
         blockwise_method = partial(
@@ -1152,6 +1170,7 @@ def dask_groupby_agg(
                 agg=agg,
                 expected_groups=None if split_out > 1 else expected_groups,
                 fill_value=fill_value,
+                reindex=reindex,
             ),
             combine=partial(combine, agg=agg),
             name=f"{name}-reduce",
@@ -1275,7 +1294,7 @@ def _convert_expected_groups_to_index(
             out.append(ex)
         elif ex is not None:
             if isbin_:
-                out.append(pd.IntervalIndex.from_arrays(ex[:-1], ex[1:]))
+                out.append(pd.IntervalIndex.from_breaks(ex))
             else:
                 if sort:
                     ex = np.sort(ex)
@@ -1511,12 +1530,7 @@ def groupby_reduce(
     #     The only way to do this consistently is mask out using min_count
     #     Consider np.sum([np.nan]) = np.nan, np.nansum([np.nan]) = 0
     if min_count is None:
-        if (
-            len(axis) < by.ndim
-            or fill_value is not None
-            # TODO: Fix npg bug where all-NaN rows are 0 instead of NaN
-            or (not has_dask and isinstance(func, str) and func in ["nanvar", "nanstd"])
-        ):
+        if len(axis) < by.ndim or fill_value is not None:
             min_count = 1
 
     # TODO: set in xarray?
@@ -1529,7 +1543,9 @@ def groupby_reduce(
     agg = _initialize_aggregation(func, array.dtype, fill_value, min_count, finalize_kwargs)
 
     if not has_dask:
-        results = _reduce_blockwise(array, by, agg, expected_groups=expected_groups, **kwargs)
+        results = _reduce_blockwise(
+            array, by, agg, expected_groups=expected_groups, reindex=reindex, **kwargs
+        )
         groups = (results["groups"],)
         result = results[agg.name]
 
