@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import itertools
 from typing import TYPE_CHECKING, Hashable, Iterable, Sequence
 
 import numpy as np
@@ -192,7 +191,7 @@ def xarray_reduce(
     if not sort:
         raise NotImplementedError
 
-    # eventually  drop the variables we are grouping by
+    # eventually drop the variables we are grouping by
     maybe_drop = [b for b in by if isinstance(b, str)]
     unindexed_dims = tuple(
         b
@@ -202,7 +201,11 @@ def xarray_reduce(
 
     by: tuple[DataArray] = tuple(obj[g] if isinstance(g, str) else g for g in by)  # type: ignore
 
-    grouper_dims = tuple(itertools.chain(*tuple(g.dims for g in by)))
+    grouper_dims = []
+    for g in by:
+        for d in g.dims:
+            if d not in grouper_dims:
+                grouper_dims.append(d)
 
     if isinstance(obj, xr.DataArray):
         ds = obj._to_temp_dataset()
@@ -210,30 +213,26 @@ def xarray_reduce(
         ds = obj
 
     ds = ds.drop_vars([var for var in maybe_drop if var in ds.variables])
+
     if dim is Ellipsis:
         dim = tuple(obj.dims)
         if by[0].name in ds.dims and not isbin[0]:
             dim = tuple(d for d in dim if d != by[0].name)
-
-    # TODO: do this for specific reductions only
-    bad_dtypes = tuple(
-        k for k in ds.variables if k not in ds.dims and ds[k].dtype.kind in ("S", "U")
-    )
+    elif dim is not None:
+        dim = _atleast_1d(dim)
+    else:
+        dim = tuple()
 
     # broadcast all variables against each other along all dimensions in `by` variables
     # don't exclude `dim` because it need not be a dimension in any of the `by` variables!
     # in the case where dim is Ellipsis, and by.ndim < obj.ndim
     # then we also broadcast `by` to all `obj.dims`
     # TODO: avoid this broadcasting
-    exclude_dims = set(ds.dims) - set(grouper_dims)
-    if dim is not None:
-        exclude_dims -= set(dim)
+    exclude_dims = tuple(d for d in ds.dims if d not in grouper_dims and d not in dim)
     ds, *by = xr.broadcast(ds, *by, exclude=exclude_dims)
 
-    if dim is None:
+    if not dim:
         dim = tuple(by[0].dims)
-    else:
-        dim = _atleast_1d(dim)
 
     if any(d not in grouper_dims and d not in obj.dims for d in dim):
         raise ValueError(f"Cannot reduce over absent dimensions {dim}.")
@@ -299,8 +298,6 @@ def xarray_reduce(
     if isinstance(obj, xr.Dataset):
         # broadcasting means the group dim gets added to ds, so we check the original obj
         for k, v in obj.data_vars.items():
-            if k in bad_dtypes:
-                continue
             is_missing_dim = not (any(d in v.dims for d in dim))
             if is_missing_dim:
                 missing_dim[k] = v
@@ -310,7 +307,7 @@ def xarray_reduce(
 
     actual = xr.apply_ufunc(
         wrapper,
-        ds.drop_vars(tuple(missing_dim) + bad_dtypes).transpose(..., *grouper_dims),
+        ds.drop_vars(tuple(missing_dim)).transpose(..., *grouper_dims),
         *by,
         input_core_dims=input_core_dims,
         # for xarray's test_groupby_duplicate_coordinate_labels
@@ -389,6 +386,8 @@ def rechunk_for_cohorts(
     labels: DataArray,
     force_new_chunk_at,
     chunksize: int | None = None,
+    ignore_old_chunks: bool = False,
+    debug: bool = False,
 ):
     """
     Rechunks array so that each new chunk contains groups that always occur together.
@@ -424,6 +423,8 @@ def rechunk_for_cohorts(
         labels,
         force_new_chunk_at=force_new_chunk_at,
         chunksize=chunksize,
+        ignore_old_chunks=ignore_old_chunks,
+        debug=debug,
     )
 
 
