@@ -53,9 +53,9 @@ ALL_FUNCS = (
     "min",
     "nanmin",
     "argmax",
-    pytest.param("nanargmax", marks=(pytest.mark.xfail,)),
+    pytest.param("nanargmax", marks=(pytest.mark.skip,)),
     "argmin",
-    pytest.param("nanargmin", marks=(pytest.mark.xfail,)),
+    pytest.param("nanargmin", marks=(pytest.mark.skip,)),
     "any",
     "all",
     pytest.param("median", marks=(pytest.mark.skip,)),
@@ -142,7 +142,7 @@ def gen_array_by(size, func):
     return array, by
 
 
-@pytest.mark.parametrize("chunks", [None, 3, 4])
+@pytest.mark.parametrize("chunks", [None, -1, 3, 4])
 @pytest.mark.parametrize("nby", [1, 2, 3])
 @pytest.mark.parametrize("size", ((12,), (12, 9)))
 @pytest.mark.parametrize("add_nan_by", [True, False])
