@@ -85,7 +85,6 @@ def test_xarray_reduce_multiple_groupers(pass_expected_groups, chunk, engine):
 
     arr = np.ones((4, 12))
     labels = np.array(["a", "a", "c", "c", "c", "b", "b", "c", "c", "b", "b", "f"])
-    labels = np.array(labels)
     labels2 = np.array([1, 2, 2, 1])
 
     da = xr.DataArray(
@@ -117,6 +116,49 @@ def test_xarray_reduce_multiple_groupers(pass_expected_groups, chunk, engine):
         actual = xarray_reduce(da, "labels", "labels2", **kwargs)
     xr.testing.assert_identical(expected, actual)
 
+    if pass_expected_groups:
+        kwargs["expected_groups"] = (expected.labels2.data, expected.labels.data)
+    with raise_if_dask_computes():
+        actual = xarray_reduce(da, "labels2", "labels", **kwargs)
+    xr.testing.assert_identical(expected.transpose("z", "labels2", "labels"), actual)
+
+
+@pytest.mark.parametrize("pass_expected_groups", [True, False])
+@pytest.mark.parametrize("chunk", (True, False))
+def test_xarray_reduce_multiple_groupers_2(pass_expected_groups, chunk, engine):
+    if not has_dask and chunk:
+        pytest.skip()
+
+    if chunk and pass_expected_groups is False:
+        pytest.skip()
+
+    arr = np.ones((2, 12))
+    labels = np.array(["a", "a", "c", "c", "c", "b", "b", "c", "c", "b", "b", "f"])
+
+    da = xr.DataArray(
+        arr, dims=("x", "y"), coords={"labels2": ("y", labels), "labels": ("y", labels)}
+    ).expand_dims(z=4)
+
+    if chunk:
+        da = da.chunk({"x": 2, "z": 1})
+
+    expected = xr.DataArray(
+        [[2, 0, 0, 0], [0, 4, 0, 0], [0, 0, 5, 0], [0, 0, 0, 1]],
+        dims=("labels", "labels2"),
+        coords={
+            "labels": ["a", "b", "c", "f"],
+            "labels2": ["a", "b", "c", "f"],
+        },
+    ).expand_dims(z=4, x=2)
+
+    kwargs = dict(func="count", engine=engine)
+    if pass_expected_groups:
+        kwargs["expected_groups"] = (expected.labels.data, expected.labels.data)
+
+    with raise_if_dask_computes():
+        actual = xarray_reduce(da, "labels", "labels2", **kwargs)
+    xr.testing.assert_identical(expected, actual)
+
 
 @requires_dask
 def test_dask_groupers_error():
